# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####
#
#
#  Authors           : Clemens Barth (Blendphys@root-1.de), ...
#
#  Homepage(Wiki)    : http://development.root-1.de/Atomic_Blender_ASE.php
#
#  Start of project              : 2012-10-14 by Clemens Barth
#  First publication in ASE      : 2012-??-??
#  Last modified                 : 2012-11-01
#
#  Acknowledgements: 
#
#  1. Igor Beinik and Frank Palmino for testing the script on the Mac.
#
#


from   ase.io import write, read
import os
import sys



# Determination of the filepaths + eventually writing an xyz file from
# the structure 
def blend_filepaths(struct):

                       
    file_blender_start    = "/tmp/blender_start_file.py"
    file_xyz_temp         = "/tmp/blender_tmp.xyz"       
    
    # For Linux
    if sys.platform == "linux2":
        dir_blender_template = (sys.path[1] + 
                                "/dist-packages/ase/blender/io_mesh_xyz/")        
    # For Mac - ASE has to be in the $HOME directory:
    # $HOME/ase
    if sys.platform == "darwin":
        dir_blender_template = (sys.path[1] +
                                "/ase/blender/io_mesh_xyz/")


    file_blender_template = [dir_blender_template + "import_xyz.py",
                             dir_blender_template + "call_xyz.py"]

    # Is it a 'Atom' structure or a xyz file?
    if type(struct) is str:
        # It's a xyz file ...
        file_xyz = struct
    else:
        # If it is a structure create a xyz file.
        file_xyz = file_xyz_temp
        write(file_xyz, struct)

    return (file_blender_start, file_blender_template, file_xyz)



# Write the Blender startfile
def blend_write_startfile(file_blender_template, 
                          file_blender_start, 
                          file_xyz, 
                          parameters):

    # Read the template file
    f = open(file_blender_template[0], 'r')
    blender_template1 = f.read()
    f.close()
    f = open(file_blender_template[1], 'r')
    blender_template2 = f.read()
    f.close()    
    

    # Create a new Blender startfile, ...
    f = open(file_blender_start, 'w')
    # ... write first the template into ...
    f.write(blender_template1)
    f.write(blender_template2)    
    # ... and put in the parameters.  
    f.write("atom_number = DEF_atom_xyz_main(" + str(parameters["use_mesh"]) + "," + 
                           str(parameters["mesh_azimuth"]) + "," +
                           str(parameters["mesh_zenith"]) + "," +
                           str(parameters["scale_ballradius"]) + "," +
                           str(parameters["atomradius"]) + "," +
                           str(parameters["scale_distances"]) + "," +
                           str(parameters["use_center"]) + "," +
                           str(parameters["use_center_all"]) + "," +
                           str(parameters["use_camera"]) + "," +
                           str(parameters["use_lamp"]) + "," + 
                           "\"" + str(file_xyz) + "\")\n")         
               
    # Search the camera in the scene and activate it.
    if parameters["use_camera"] == True:
        f.write("cameras = []\n" +    
                "for obj in bpy.context.scene.objects:\n"
                "    if obj.type == \"CAMERA\":\n"
                "        cameras.append(obj)\n\n"
                "bpy.context.scene.camera = cameras[0]\n\n")     

    # Load the frames?
    if parameters["load_frames"] == True:
        f.write("DEF_atom_xyz_build_frames(" + str(parameters["frame_delta"]) + 
                ", "+ str(parameters["frame_skip"]) + ")\n\n")

    # Save the scene into a blend file 'file_blend'
    if parameters["blender_save_file"] != "":    
        f.write("bpy.ops.wm.save_as_mainfile(filepath=\"" + 
                                             parameters["blender_save_file"] 
                                             + "\")\n\n")
    
    # Close Blender after having opened? This is important for the rendering
    # without the UI opened.
    if parameters["blender_close"] == True:
        f.write("bpy.ops.wm.quit_blender()\n\n")        
                                                                                    
    f.close()



def blend_show(struct, **kwargs):

    # Default values
    parameters = {"use_mesh"         : False, 
                  "mesh_azimuth"     : 32,
                  "mesh_zenith"      : 32,
                  "scale_ballradius" : 1.0,
                  "atomradius"       : "0",
                  "scale_distances"  : 1.0,
                  "use_center"       : True,
                  "use_center_all"   : True,
                  "use_camera"       : True,
                  "use_lamp"         : True,
                  "load_frames"      : False,
                  "frame_delta"      : 1,
                  "frame_skip"       : 0,
                  "blender_close"    : False,
                  "blender_save_file": ""}  

    # Update values
    parameters.update(kwargs)
           
    # Get all file paths.             
    (file_blender_start, 
     file_blender_template, 
     file_xyz)               = blend_filepaths(struct)

    # Write the Blender start file, ...    
    blend_write_startfile(file_blender_template, 
                          file_blender_start, 
                          file_xyz, 
                          parameters)

    # For Linux
    if sys.platform == "linux2":
        os_string = "blender -P " + file_blender_start + " &"
        
    # For Mac
    if sys.platform == "darwin":
        os_string = ("/Applications/Blender/blender.app/Contents/MacOS/blender -P " 
                      + file_blender_start + " &")

    #print os_string
    os.system(os_string)  






def blend_render(struct, file_blend, file_movie, **kwargs):

    # Default values
    parameters = {"use_mesh"         : False, 
                  "mesh_azimuth"     : 32,
                  "mesh_zenith"      : 32,
                  "scale_ballradius" : 1.0,
                  "atomradius"       : "0",
                  "scale_distances"  : 1.0,
                  "use_center"       : True,
                  "use_center_all"   : True,
                  "use_camera"       : True,
                  "use_lamp"         : True,
                  "load_frames"      : False,
                  "frame_delta"      : 1,
                  "frame_skip"       : 0,
                  "blender_close"    : False,
                  "blender_save_file": "",                                                              
                  "image_format"     : "PNG",
                  "frame_start"      : 1,
                  "frame_end"        : 1}  

    # Update values
    parameters.update(kwargs)
                 
    # Get all file paths.             
    (file_blender_start, 
     file_blender_template, 
     file_xyz)               = blend_filepaths(struct)
 
    # Put the Blender file path into the correct variable.
    parameters["blender_save_file"] = file_blend
          
    # Write the Blender startfile ...    
    blend_write_startfile(file_blender_template, 
                          file_blender_start, 
                          file_xyz, 
                          parameters)                           
                                
    # Execute Blender with the start file in the terminal. What happens is that
    # Blender reads the xyz file, adjusts the scene and saves the whole into 
    # the Blender file 'file_blend'. After all, Blender closes 
    # for 'blender_close = True'. 

    # For Linux
    if sys.platform == "linux2":   
        os_string = "blender -P " + file_blender_start

    # For Mac - Blender has to be in:
    if sys.platform == "darwin":
        os_string = "/Applications/Blender/blender.app/Contents/MacOS/blender -P " + file_blender_start
        
    #print os_string        
    os.system(os_string)
                
    # Now, call Blender again for rendering an image. The Blender GUI will not 
    # be opended.
    
    # For Linux
    if sys.platform == "linux2":   
        os_string = ("blender -b " + 
                 file_blend + " " +
                 "-x 1 -o //" +
                 file_movie + "_ " +
                 "-F "  +     parameters["image_format"] + 
                 " -s " + str(parameters["frame_start"]) +
                 " -e " + str(parameters["frame_end"])   +
                 " -a")  

    # For Mac
    if sys.platform == "darwin":
        os_string = ("/Applications/Blender/blender.app/Contents/MacOS/blender -b " + 
                 file_blend + " " +
                 "-x 1 -o //" +
                 file_movie + "_ " +
                 "-F "  +     parameters["image_format"] + 
                 " -s " + str(parameters["frame_start"]) +
                 " -e " + str(parameters["frame_end"])   +
                 " -a")  
                 
                 
    print os_string                             
    os.system(os_string)


    
   

